@@ -8,6 +8,7 @@ nimblePreevaluationFunctionNames <- c('+',
                                       'exp',
                                       'log',
                                       'pow',
+                                      'pow_int',
                                       '^',
                                       '%%',
                                       'equals',
@@ -336,7 +337,8 @@ BUGSdeclClass$methods(
                  context,
                  nimFunNames,
                  unknownIndexDeclInfo = NULL,
-                 contextID = NULL) {
+                 contextID = NULL,
+                 buildDerivs = FALSE) {
     ## sets the field symbolicparentNodes
             symbolicParentNodes <<-
                 unique(
@@ -345,7 +347,8 @@ BUGSdeclClass$methods(
                                            context$indexVarExprs,
                                            nimFunNames,
                                            contextID = contextID,
-                                           envir = envir)
+                                           envir = envir,
+                                           buildDerivs = buildDerivs)
                 ) 
         }
 )
@@ -406,7 +409,8 @@ BUGSdeclClass$methods(
     genReplacedTargetValueAndParentInfo = function(constantsNamesList,
                                                    context,
                                                    nimFunNames,
-                                                   contextID = NULL) {
+                                                   contextID = NULL,
+                                                   buildDerivs = FALSE) {
         ## This assumes codeReplaced is there.
         ## Generate hasBracket info:
         targetExprReplaced <<- codeReplaced[[2]]
@@ -425,7 +429,8 @@ BUGSdeclClass$methods(
                                          replacementNameExprs),
                                        nimFunNames,
                                        contextID = contextID,
-                                       envir = envir)
+                                       envir = envir,
+                                       buildDerivs = buildDerivs)
             )
     if(!nimbleOptions()$allowDynamicIndexing) {
         rhsVars <<-
@@ -502,13 +507,15 @@ BUGSdeclClass$methods(
 BUGSdeclClass$methods(
     genReplacementsAndCodeReplaced = function(constantsNamesList,
                                               context,
-                                              nimFunNames) {
+                                              nimFunNames,
+                                              checkAD = FALSE) {
         replacementsAndCode <-
             genReplacementsAndCodeRecurse(code,
                                           c(constantsNamesList,
                                             context$indexVarExprs),
                                           nimFunNames,
-                                          envir = envir)
+                                          envir = envir,
+                                          checkAD = checkAD)
         replacements <<- replacementsAndCode$replacements
         codeReplaced <<- replacementsAndCode$codeReplaced
         
@@ -636,7 +643,8 @@ getSymbolicParentNodes <- function(code,
                                    nimbleFunctionNames = list(),
                                    addDistNames = FALSE,
                                    contextID = NULL,
-                                   envir = .GlobalEnv) {
+                                   envir = .GlobalEnv,
+                                   buildDerivs = FALSE) {
     if(addDistNames)
         nimbleFunctionNames <- c(nimbleFunctionNames,
                                  getAllDistributionsInfo('namesExprList'))
@@ -645,11 +653,14 @@ getSymbolicParentNodes <- function(code,
                                          indexNames,
                                          nimbleFunctionNames,
                                          contextID,
-                                         envir)
+                                         envir,
+                                         buildDerivs = buildDerivs)
     return(ans$code)
 }
 
-getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames = list(), nimbleFunctionNames = list(), contextID = NULL, envir = .GlobalEnv) {
+getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames = list(),
+                                          nimbleFunctionNames = list(), contextID = NULL,
+                                          envir = .GlobalEnv, buildDerivs = FALSE) {
     ## This takes as input some code and returns the variables in it.
     ## It expects one line of code, not a '{' expression.
     ##
@@ -740,7 +751,8 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
                                                          indexNames,
                                                          nimbleFunctionNames,
                                                          contextID,
-                                                         envir)
+                                                         envir,
+                                                         buildDerivs = buildDerivs)
                        )
             ## unpack the codes returned from recursion
             contentsCode <-
@@ -764,7 +776,8 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
                                               indexNames,
                                               nimbleFunctionNames,
                                               contextID,
-                                              envir)
+                                              envir,
+                                              buildDerivs = buildDerivs)
             
             ## error if it looks like mu[i][j] where i is a for-loop index
             if(variable$hasIndex)
@@ -808,11 +821,15 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
                                 hasIndex = any(contentsHasIndex)))
                 } else { ## non-replaceable indices are dynamic indices (or constant vectors, which are not allowed)
                     if(!nimbleOptions()$allowDynamicIndexing) {
-                        warning("It appears you are trying to use dynamic indexing (i.e., the index of a variable is determined by something that is not a constant) in: ",
+                        message("  [Note] It appears you are trying to use dynamic indexing (i.e., the index of a variable is determined by something that is not a constant) in: `",
                                 safeDeparse(code),
-                                ". This is now allowed as of version 0.6-6 (as an optional beta feature) and by default as of version 0.6-7. Please set 'nimbleOptions(allowDynamicIndexing = TRUE)' and report any issues to the NIMBLE users group.")
+                                "`. Please set `nimbleOptions(allowDynamicIndexing = TRUE)`.")
                         dynamicIndexParent <- code[[2]]
                     } else {
+                        if(isTRUE(nimbleOptions("doADerrorTraps")))
+                          if(isTRUE(buildDerivs))
+                            message("  [Warning] Derivatives cannot currently be built for models that include dynamic indexing (found in `", safeDeparse(code), "`).  Please set 'nimbleOptions(buildDerivs = FALSE)' to proceed with this model.")
+                      
                         if(any(
                             sapply(contentsCode,
                                    detectNonscalarIndex))
@@ -853,7 +870,8 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
                                                           indexNames,
                                                           nimbleFunctionNames,
                                                           contextID,
-                                                          envir)
+                                                          envir,
+                                                          buildDerivs = buildDerivs)
                     )
                 else ## foo(x): recurse on x
                     contents <- lapply(
@@ -864,7 +882,8 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
                                                           indexNames,
                                                           nimbleFunctionNames,
                                                           contextID,
-                                                          envir)
+                                                          envir,
+                                                          buildDerivs = buildDerivs)
                     )
                 ## unpack results of recursion
                 contentsCode <- unlist(
@@ -933,7 +952,8 @@ genReplacementsAndCodeRecurse <- function(code,
                                           nimbleFunctionNames,
                                           replaceVariableLHS = TRUE,
                                           debug = FALSE,
-                                          envir = .GlobalEnv) {
+                                          envir = .GlobalEnv,
+                                          checkAD = FALSE) {
     if(debug) browser()
     if(is.numeric(code) || is.logical(code) ||
        (nimbleOptions()$allowDynamicIndexing &&
@@ -970,7 +990,8 @@ genReplacementsAndCodeRecurse <- function(code,
                                                   constAndIndexNames,
                                                   nimbleFunctionNames,
                                                   debug = debug,
-                                                  envir = envir)
+                                                  envir = envir,
+                                                  checkAD = checkAD)
             )
             contentsCodeReplaced <-
                 lapply(contents, function(x) x$codeReplaced)
@@ -984,7 +1005,8 @@ genReplacementsAndCodeRecurse <- function(code,
                                                   constAndIndexNames,
                                                   nimbleFunctionNames,
                                                   debug = debug,
-                                                  envir = envir)
+                                                  envir = envir,
+                                                  checkAD = checkAD)
                 if(variable$replaceable &&
                    all(contentsReplaceable))
                     return(replaceAllCodeSuccessfully(code))
@@ -1007,7 +1029,8 @@ genReplacementsAndCodeRecurse <- function(code,
                                                           constAndIndexNames,
                                                           nimbleFunctionNames,
                                                           replaceVariableLHS = FALSE, debug,
-                                                          envir = envir)
+                                                          envir = envir,
+                                                          checkAD = checkAD)
                         ),
                         lapply(
                             code[-c(1,2)],
@@ -1016,7 +1039,8 @@ genReplacementsAndCodeRecurse <- function(code,
                                                               constAndIndexNames,
                                                               nimbleFunctionNames,
                                                               debug = debug,
-                                                              envir = envir))
+                                                              envir = envir,
+                                                              checkAD = checkAD))
                     )
             } else {
                 contents <- lapply(
@@ -1026,7 +1050,8 @@ genReplacementsAndCodeRecurse <- function(code,
                                                       constAndIndexNames,
                                                       nimbleFunctionNames,
                                                       debug = debug,
-                                                      envir = envir))
+                                                      envir = envir,
+                                                      checkAD = checkAD))
             }
             contentsCodeReplaced <- lapply(contents, function(x) x$codeReplaced)
             contentsReplacements <- lapply(contents, function(x) x$replacements)
@@ -1073,7 +1098,18 @@ genReplacementsAndCodeRecurse <- function(code,
                         '\" has non-replaceable node values as arguments.  Must be a nimble function.')
                  )
         if(isRfunction & allContentsReplaceable)
-            return(replaceAllCodeSuccessfully(code))
+          return(replaceAllCodeSuccessfully(code))
+
+        if(checkAD && isTRUE(nimbleOptions('doADerrorTraps'))) {
+          thisCallName <- safeDeparse(code[[1]])
+          thisCheck <- try(any(thisCallName==fxnsNotAllowedInAD))
+          if(isTRUE(thisCheck)) {
+            message("  [Warning] function ", thisCallName, " is not supported for derivatives.\n",
+                    "This model will not compile. If you don't need derivatives, use buildDerivs=FALSE.\n",
+                    "To silence this message, do nimbleOptions(doADerrorTraps=FALSE).")
+          }
+        }
+
         return(replaceWhatWeCan(code,
                                 contentsCodeReplaced,
                                 contentsReplacements,
