@@ -77,7 +77,8 @@ cppNamespace <- setRefClass('cppNamespace',
 nimbleCppInheritanceInfo <- list(
     Values = c('NamedObjects'),
     ModelBase = c('NamedObjects'),
-    nodeFun = c('NamedObjects')
+    nodeFun = c('NamedObjects'),
+    derivNodeFun = c('nodeFun')
 )
 
 ## C++ class object.
@@ -230,6 +231,19 @@ cppCodeBlock <- setRefClass('cppCodeBlock',
                                           cppADCode = 'ANY', generatorSymTab = 'ANY'),
                             methods = list(
                                 generate = function(indent = '', ...) {
+                                    ## TRUE was an original possible value.
+                                    ## It was deprecated for the new value, 2L.
+                                    ## This originally was version 2. Now it is the version.
+                                    ## if(isTRUE(cppADCode)){
+                                    ##     oldCppADCode <- nimbleUserNamespace$cppADCode
+                                    ##     nimbleUserNamespace$cppADCode <- TRUE
+                                    ##     on.exit(nimbleUserNamespace$cppADCode <- oldCppADCode)
+                                    ## }
+                                    if(identical(cppADCode, 2L)){
+                                        oldCppADCode <- nimbleUserNamespace$cppADCode
+                                        nimbleUserNamespace$cppADCode <- 2L
+                                        on.exit(nimbleUserNamespace$cppADCode <- oldCppADCode)
+                                    }
                                     if(inherits(typeDefs, 'uninitializedField')) typeDefs <<- list()
                                     typeDefsToUse <- if(inherits(typeDefs, 'symbolTable')) typeDefs$symbols else typeDefs
                                     if(length(typeDefsToUse) > 0) {
@@ -245,9 +259,7 @@ cppCodeBlock <- setRefClass('cppCodeBlock',
                                         if(inherits(generatorSymTab, 'symbolTable')) useSymTab <- generatorSymTab
                                         else if(!inherits(objectDefs, 'symbolTable')) stop('Error, with exprClass code in the cppCodeBlock, must have objectDefs be a symbolTable')
                                         else useSymTab <- objectDefs
-                                        if(identical(cppADCode, TRUE)) recurseSetCppADExprs(code, TRUE) 
-                                      outputCppCode <- c(outputCppCode, nimGenerateCpp(code, useSymTab, indent = ' ', showBracket = FALSE))
-                                        if(identical(cppADCode, TRUE)) recurseSetCppADExprs(code, FALSE) 
+                                        outputCppCode <- c(outputCppCode, nimGenerateCpp(code, useSymTab, indent = ' ', showBracket = FALSE))
                                     } else {
                                         outputCppCode <- c(outputCppCode, outputCppParseTree2(code, indent))
                                     }
@@ -296,12 +308,15 @@ cppFunctionDef <- setRefClass('cppFunctionDef',
                                           }
                                            return(outputCode) 
                                       } else {
-                                          if(inherits(code$code, 'uninitializedField')) {
+                                          code_is_empty <- inherits(code$code, 'uninitializedField')
+                                          if(code_is_empty) {
                                               ## There is no code. This can occur for a nimbleFunctionVirtual, which is an abstract base class.
-                                              return(character(0))
+                                              if(abstract)
+                                                  return(character(0))
                                           }
-                                          c(paste(generateFunctionHeader(returnType, name, argsToUse, scopes, template, static = FALSE, ...), if(const) ' const ' else character(0), '{'),
-                                            code$generate(...),
+                                          c(paste(generateFunctionHeader(returnType, name, argsToUse, scopes, template, static = FALSE, ...),
+                                                  if(const) ' const ' else character(0), '{'),
+                                            if(!code_is_empty) code$generate(...) else '',
                                             list('}'))
                                       }
                                   }
